@@ -1,32 +1,36 @@
 """IPython terminal interface using prompt_toolkit"""
 
-import asyncio
 import os
 import sys
+import inspect
 from warnings import warn
-from typing import Union as UnionType
+from typing import Union as UnionType, Optional
 
 from IPython.core.async_helpers import get_asyncio_loop
 from IPython.core.interactiveshell import InteractiveShell, InteractiveShellABC
 from IPython.utils.py3compat import input
+from IPython.utils.PyColorize import theme_table
 from IPython.utils.terminal import toggle_set_term_title, set_term_title, restore_term_title
 from IPython.utils.process import abbrev_cwd
 from traitlets import (
+    Any,
     Bool,
-    Unicode,
     Dict,
+    Enum,
+    Float,
+    Instance,
     Integer,
     List,
-    observe,
-    Instance,
     Type,
-    default,
-    Enum,
+    Unicode,
     Union,
-    Any,
+    default,
+    observe,
     validate,
-    Float,
+    DottedObjectName,
 )
+from traitlets.utils.importstring import import_item
+
 
 from prompt_toolkit.auto_suggest import AutoSuggestFromHistory
 from prompt_toolkit.enums import DEFAULT_BUFFER, EditingMode
@@ -39,11 +43,8 @@ from prompt_toolkit.patch_stdout import patch_stdout
 from prompt_toolkit.shortcuts import PromptSession, CompleteStyle, print_formatted_text
 from prompt_toolkit.styles import DynamicStyle, merge_styles
 from prompt_toolkit.styles.pygments import style_from_pygments_cls, style_from_pygments_dict
-from prompt_toolkit import __version__ as ptk_version
-
 from pygments.styles import get_style_by_name
 from pygments.style import Style
-from pygments.token import Token
 
 from .debugger import TerminalPdb, Pdb
 from .magics import TerminalMagics
@@ -52,6 +53,7 @@ from .prompts import Prompts, ClassicPrompts, RichPromptDisplayHook
 from .ptutils import IPythonPTCompleter, IPythonPTLexer
 from .shortcuts import (
     KEY_BINDINGS,
+    UNASSIGNED_ALLOWED_COMMANDS,
     create_ipython_shortcuts,
     create_identifier,
     RuntimeBinding,
@@ -63,26 +65,38 @@ from .shortcuts.auto_suggest import (
     AppendAutoSuggestionInAnyLine,
 )
 
-PTK3 = ptk_version.startswith('3.')
 
-
-class _NoStyle(Style): pass
+class _NoStyle(Style):
+    pass
 
 
 
-_style_overrides_light_bg = {
-            Token.Prompt: '#ansibrightblue',
-            Token.PromptNum: '#ansiblue bold',
-            Token.OutPrompt: '#ansibrightred',
-            Token.OutPromptNum: '#ansired bold',
-}
+def _backward_compat_continuation_prompt_tokens(
+    method, width: int, *, lineno: int, wrap_count: int
+):
+    """
+    Sagemath use custom prompt and we broke them in 8.19.
+
+    make sure to pass only width if method only support width
+    """
+    sig = inspect.signature(method)
+    extra = {}
+    params = inspect.signature(method).parameters
+    if "lineno" in inspect.signature(method).parameters or any(
+        [p.kind == p.VAR_KEYWORD for p in sig.parameters.values()]
+    ):
+        extra["lineno"] = lineno
+    if "line_number" in inspect.signature(method).parameters or any(
+        [p.kind == p.VAR_KEYWORD for p in sig.parameters.values()]
+    ):
+        extra["line_number"] = lineno
+
+    if "wrap_count" in inspect.signature(method).parameters or any(
+        [p.kind == p.VAR_KEYWORD for p in sig.parameters.values()]
+    ):
+        extra["wrap_count"] = wrap_count
+    return method(width, **extra)
 
-_style_overrides_linux = {
-            Token.Prompt: '#ansibrightgreen',
-            Token.PromptNum: '#ansigreen bold',
-            Token.OutPrompt: '#ansibrightred',
-            Token.OutPromptNum: '#ansired bold',
-}
 
 def get_default_editor():
     try:
@@ -96,7 +110,8 @@ def get_default_editor():
     if os.name == 'posix':
         return 'vi'  # the only one guaranteed to be there!
     else:
-        return 'notepad' # same in Windows!
+        return "notepad"  # same in Windows!
+
 
 # conservatively check for tty
 # overridden streams can result in things like:
@@ -188,17 +203,24 @@ class PtkHistoryAdapter(History):
 class TerminalInteractiveShell(InteractiveShell):
     mime_renderers = Dict().tag(config=True)
 
-    space_for_menu = Integer(6, help='Number of line at the bottom of the screen '
-                                     'to reserve for the tab completion menu, '
-                                     'search history, ...etc, the height of '
-                                     'these menus will at most this value. '
-                                     'Increase it is you prefer long and skinny '
-                                     'menus, decrease for short and wide.'
-                            ).tag(config=True)
+    min_elide = Integer(
+        30, help="minimum characters for filling with ellipsis in file completions"
+    ).tag(config=True)
+    space_for_menu = Integer(
+        6,
+        help="Number of line at the bottom of the screen "
+        "to reserve for the tab completion menu, "
+        "search history, ...etc, the height of "
+        "these menus will at most this value. "
+        "Increase it is you prefer long and skinny "
+        "menus, decrease for short and wide.",
+    ).tag(config=True)
 
     pt_app: UnionType[PromptSession, None] = None
     auto_suggest: UnionType[
-        AutoSuggestFromHistory, NavigableAutoSuggestFromHistory, None
+        AutoSuggestFromHistory,
+        NavigableAutoSuggestFromHistory,
+        None,
     ] = None
     debugger_history = None
 
@@ -209,12 +231,18 @@ class TerminalInteractiveShell(InteractiveShell):
     simple_prompt = Bool(_use_simple_prompt,
         help="""Use `raw_input` for the REPL, without completion and prompt colors.
 
-            Useful when controlling IPython as a subprocess, and piping STDIN/OUT/ERR. Known usage are:
-            IPython own testing machinery, and emacs inferior-shell integration through elpy.
+            Useful when controlling IPython as a subprocess, and piping
+            STDIN/OUT/ERR. Known usage are: IPython's own testing machinery,
+            and emacs' inferior-python subprocess (assuming you have set
+            `python-shell-interpreter` to "ipython") available through the
+            built-in `M-x run-python` and third party packages such as elpy.
 
             This mode default to `True` if the `IPY_TEST_SIMPLE_PROMPT`
-            environment variable is set, or the current terminal is not a tty."""
-            ).tag(config=True)
+            environment variable is set, or the current terminal is not a tty.
+            Thus the Default value reported in --help-all, or config will often
+            be incorrectly reported.
+            """,
+    ).tag(config=True)
 
     @property
     def debugger_cls(self):
@@ -276,9 +304,12 @@ class TerminalInteractiveShell(InteractiveShell):
 
     # We don't load the list of styles for the help string, because loading
     # Pygments plugins takes time and can cause unexpected errors.
-    highlighting_style = Union([Unicode('legacy'), Type(klass=Style)],
-        help="""The name or class of a Pygments style to use for syntax
-        highlighting. To see available styles, run `pygmentize -L styles`."""
+    highlighting_style = Union(
+        [Unicode("legacy"), Type(klass=Style)],
+        help="""Deprecated, and has not effect, use IPython themes
+
+        The name or class of a Pygments style to use for syntax
+        highlighting. To see available styles, run `pygmentize -L styles`.""",
     ).tag(config=True)
 
     @validate('editing_mode')
@@ -293,7 +324,6 @@ class TerminalInteractiveShell(InteractiveShell):
 
         return self.editing_mode
 
-
     @observe('editing_mode')
     def _editing_mode(self, change):
         if self.pt_app:
@@ -317,12 +347,17 @@ class TerminalInteractiveShell(InteractiveShell):
     @observe('highlighting_style')
     @observe('colors')
     def _highlighting_style_changed(self, change):
-        self.refresh_style()
+        assert change.new == change.new.lower()
+        if change.new != "legacy":
+            warn(
+                "highlighting_style is deprecated since 9.0 and have no effect, use themeing."
+            )
+            return
 
     def refresh_style(self):
-        self._style = self._make_style_from_name_or_cls(self.highlighting_style)
-
+        self._style = self._make_style_from_name_or_cls("legacy")
 
+    # TODO: deprecate this
     highlighting_style_overrides = Dict(
         help="Override highlighting format for specific tokens"
     ).tag(config=True)
@@ -400,8 +435,66 @@ class TerminalInteractiveShell(InteractiveShell):
         "Default is `'NavigableAutoSuggestFromHistory`'.",
         allow_none=True,
     ).tag(config=True)
+    _autosuggestions_provider: Any
 
-    def _set_autosuggestions(self, provider):
+    llm_constructor_kwargs = Dict(
+        {},
+        help="""
+        Extra arguments to pass to `llm_provider_class` constructor.
+
+        This is used to – for example – set the `model_id`""",
+    ).tag(config=True)
+
+    llm_prefix_from_history = DottedObjectName(
+        "input_history",
+        help="""\
+    Fully Qualifed name of a function that takes an IPython history manager and
+    return a prefix to pass the llm provider in addition to the current buffer
+    text.
+
+    You can use:
+
+     - no_prefix
+     - input_history
+
+    As default value. `input_history` (default),  will use all the input history
+    of current IPython session
+
+    """,
+    ).tag(config=True)
+    _llm_prefix_from_history: Any
+
+    @observe("llm_prefix_from_history")
+    def _llm_prefix_from_history_changed(self, change):
+        name = change.new
+        self._llm_prefix_from_history = name
+        self._set_autosuggestions()
+
+    llm_provider_class = DottedObjectName(
+        None,
+        allow_none=True,
+        help="""\
+        Provisional:
+            This is a provisional API in IPython 8.32, before stabilisation
+            in 9.0, it may change without warnings.
+
+        class to use for the `NavigableAutoSuggestFromHistory` to request
+        completions from a LLM, this should inherit from
+        `jupyter_ai_magics:BaseProvider` and implement
+        `stream_inline_completions`
+    """,
+    ).tag(config=True)
+    _llm_provider_class: Any = None
+
+    @observe("llm_provider_class")
+    def _llm_provider_class_changed(self, change):
+        provider_class = change.new
+        self._llm_provider_class = provider_class
+        self._set_autosuggestions()
+
+    def _set_autosuggestions(self, provider=None):
+        if provider is None:
+            provider = self.autosuggestions_provider
         # disconnect old handler
         if self.auto_suggest and isinstance(
             self.auto_suggest, NavigableAutoSuggestFromHistory
@@ -412,7 +505,38 @@ class TerminalInteractiveShell(InteractiveShell):
         elif provider == "AutoSuggestFromHistory":
             self.auto_suggest = AutoSuggestFromHistory()
         elif provider == "NavigableAutoSuggestFromHistory":
+            # LLM stuff are all Provisional in 8.32
+            if self._llm_provider_class:
+
+                def init_llm_provider():
+                    llm_provider_constructor = import_item(self._llm_provider_class)
+                    return llm_provider_constructor(**self.llm_constructor_kwargs)
+
+            else:
+                init_llm_provider = None
             self.auto_suggest = NavigableAutoSuggestFromHistory()
+            # Provisinal in 8.32
+            self.auto_suggest._init_llm_provider = init_llm_provider
+
+            name = self.llm_prefix_from_history
+
+            if name == "no_prefix":
+
+                def no_prefix(history_manager):
+                    return ""
+
+                fun = no_prefix
+
+            elif name == "input_history":
+
+                def input_history(history_manager):
+                    return "\n".join([s[2] for s in history_manager.get_range()]) + "\n"
+
+                fun = input_history
+
+            else:
+                fun = import_item(name)
+            self.auto_suggest._llm_prefixer = fun
         else:
             raise ValueError("No valid provider.")
         if self.pt_app:
@@ -444,23 +568,23 @@ class TerminalInteractiveShell(InteractiveShell):
                 "create": Bool(False),
             },
         ),
-        help="""Add, disable or modifying shortcuts.
+        help="""
+        Add, disable or modifying shortcuts.
 
         Each entry on the list should be a dictionary with ``command`` key
         identifying the target function executed by the shortcut and at least
         one of the following:
 
-        - ``match_keys``: list of keys used to match an existing shortcut,
-        - ``match_filter``: shortcut filter used to match an existing shortcut,
-        - ``new_keys``: list of keys to set,
-        - ``new_filter``: a new shortcut filter to set
+          - ``match_keys``: list of keys used to match an existing shortcut,
+          - ``match_filter``: shortcut filter used to match an existing shortcut,
+          - ``new_keys``: list of keys to set,
+          - ``new_filter``: a new shortcut filter to set
 
         The filters have to be composed of pre-defined verbs and joined by one
         of the following conjunctions: ``&`` (and), ``|`` (or), ``~`` (not).
         The pre-defined verbs are:
 
-        {}
-
+        {filters}
 
         To disable a shortcut set ``new_keys`` to an empty list.
         To add a shortcut add key ``create`` with value ``True``.
@@ -475,8 +599,27 @@ class TerminalInteractiveShell(InteractiveShell):
         shortcuts) can be modified or disabled. The full list of shortcuts,
         command identifiers and filters is available under
         :ref:`terminal-shortcuts-list`.
+
+        Here is an example:
+
+        .. code::
+
+            c.TerminalInteractiveShell.shortcuts = [
+               {{
+                   "new_keys": ["c-q"],
+                   "command": "prompt_toolkit:named_commands.capitalize_word",
+                   "create": True,
+               }},
+               {{
+                   "new_keys": ["c-j"],
+                   "command": "prompt_toolkit:named_commands.beginning_of_line",
+                   "create": True,
+               }},
+            ]
+
+
         """.format(
-            "\n        ".join([f"- `{k}`" for k in KEYBINDING_FILTERS])
+            filters="\n        ".join([f"  - ``{k}``" for k in KEYBINDING_FILTERS])
         ),
     ).tag(config=True)
 
@@ -489,19 +632,25 @@ class TerminalInteractiveShell(InteractiveShell):
         # rebuild the bindings list from scratch
         key_bindings = create_ipython_shortcuts(self)
 
-        # for now we only allow adding shortcuts for commands which are already
-        # registered; this is a security precaution.
-        known_commands = {
+        # for now we only allow adding shortcuts for a specific set of
+        # commands; this is a security precution.
+        allowed_commands = {
             create_identifier(binding.command): binding.command
             for binding in KEY_BINDINGS
         }
+        allowed_commands.update(
+            {
+                create_identifier(command): command
+                for command in UNASSIGNED_ALLOWED_COMMANDS
+            }
+        )
         shortcuts_to_skip = []
         shortcuts_to_add = []
 
         for shortcut in user_shortcuts:
             command_id = shortcut["command"]
-            if command_id not in known_commands:
-                allowed_commands = "\n - ".join(known_commands)
+            if command_id not in allowed_commands:
+                allowed_commands = "\n - ".join(allowed_commands)
                 raise ValueError(
                     f"{command_id} is not a known shortcut command."
                     f" Allowed commands are: \n - {allowed_commands}"
@@ -525,7 +674,7 @@ class TerminalInteractiveShell(InteractiveShell):
             new_keys = shortcut.get("new_keys", None)
             new_filter = shortcut.get("new_filter", None)
 
-            command = known_commands[command_id]
+            command = allowed_commands[command_id]
 
             creating_new = shortcut.get("create", False)
             modifying_existing = not creating_new and (
@@ -567,12 +716,14 @@ class TerminalInteractiveShell(InteractiveShell):
                     RuntimeBinding(
                         command,
                         keys=new_keys or old_keys,
-                        filter=filter_from_string(new_filter)
-                        if new_filter is not None
-                        else (
-                            old_filter
-                            if old_filter is not None
-                            else filter_from_string("always")
+                        filter=(
+                            filter_from_string(new_filter)
+                            if new_filter is not None
+                            else (
+                                old_filter
+                                if old_filter is not None
+                                else filter_from_string("always")
+                            )
                         ),
                     )
                 )
@@ -588,6 +739,17 @@ class TerminalInteractiveShell(InteractiveShell):
         help="Display the current vi mode (when using vi editing mode)."
     ).tag(config=True)
 
+    prompt_line_number_format = Unicode(
+        "",
+        help="The format for line numbering, will be passed `line` (int, 1 based)"
+        " the current line number and `rel_line` the relative line number."
+        " for example to display both you can use the following template string :"
+        " c.TerminalInteractiveShell.prompt_line_number_format='{line: 4d}/{rel_line:+03d} | '"
+        " This will display the current line number, with leading space and a width of at least 4"
+        " character, as well as the relative line number 0 padded and always with a + or - sign."
+        " Note that when using Emacs mode the prompt of the first line may not update.",
+    ).tag(config=True)
+
     @observe('term_title')
     def init_term_title(self, change=None):
         # Enable or disable the terminal title.
@@ -603,8 +765,13 @@ class TerminalInteractiveShell(InteractiveShell):
 
     def init_display_formatter(self):
         super(TerminalInteractiveShell, self).init_display_formatter()
-        # terminal only supports plain text
-        self.display_formatter.active_types = ["text/plain"]
+        # terminal only supports plain text if not explicitly configured
+        config = self.display_formatter._trait_values["config"]
+        if not (
+            "DisplayFormatter" in config
+            and "active_types" in config["DisplayFormatter"]
+        ):
+            self.display_formatter.active_types = ["text/plain"]
 
     def init_prompt_toolkit_cli(self):
         if self.simple_prompt:
@@ -613,10 +780,13 @@ class TerminalInteractiveShell(InteractiveShell):
             def prompt():
                 prompt_text = "".join(x[1] for x in self.prompts.in_prompt_tokens())
                 lines = [input(prompt_text)]
-                prompt_continuation = "".join(x[1] for x in self.prompts.continuation_prompt_tokens())
-                while self.check_complete('\n'.join(lines))[0] == 'incomplete':
-                    lines.append( input(prompt_continuation) )
-                return '\n'.join(lines)
+                prompt_continuation = "".join(
+                    x[1] for x in self.prompts.continuation_prompt_tokens()
+                )
+                while self.check_complete("\n".join(lines))[0] == "incomplete":
+                    lines.append(input(prompt_continuation))
+                return "\n".join(lines)
+
             self.prompt_for_code = prompt
             return
 
@@ -626,12 +796,12 @@ class TerminalInteractiveShell(InteractiveShell):
         # Pre-populate history from IPython's history database
         history = PtkHistoryAdapter(self)
 
-        self._style = self._make_style_from_name_or_cls(self.highlighting_style)
-        self.style = DynamicStyle(lambda: self._style)
+        self.refresh_style()
+        ptk_s = DynamicStyle(lambda: self._style)
 
         editing_mode = getattr(EditingMode, self.editing_mode.upper())
 
-        self.pt_loop = asyncio.new_event_loop()
+        self._use_asyncio_inputhook = False
         self.pt_app = PromptSession(
             auto_suggest=self.auto_suggest,
             editing_mode=editing_mode,
@@ -639,7 +809,7 @@ class TerminalInteractiveShell(InteractiveShell):
             history=history,
             completer=IPythonPTCompleter(shell=self),
             enable_history_search=self.enable_history_search,
-            style=self.style,
+            style=ptk_s,
             include_default_pygments_style=False,
             mouse_support=self.mouse_support,
             enable_open_in_editor=self.extra_open_editor_shortcuts,
@@ -656,64 +826,28 @@ class TerminalInteractiveShell(InteractiveShell):
 
         We need that to add style for prompt ... etc.
         """
-        style_overrides = {}
-        if name_or_cls == 'legacy':
-            legacy = self.colors.lower()
-            if legacy == 'linux':
-                style_cls = get_style_by_name('monokai')
-                style_overrides = _style_overrides_linux
-            elif legacy == 'lightbg':
-                style_overrides = _style_overrides_light_bg
-                style_cls = get_style_by_name('pastie')
-            elif legacy == 'neutral':
-                # The default theme needs to be visible on both a dark background
-                # and a light background, because we can't tell what the terminal
-                # looks like. These tweaks to the default theme help with that.
-                style_cls = get_style_by_name('default')
-                style_overrides.update({
-                    Token.Number: '#ansigreen',
-                    Token.Operator: 'noinherit',
-                    Token.String: '#ansiyellow',
-                    Token.Name.Function: '#ansiblue',
-                    Token.Name.Class: 'bold #ansiblue',
-                    Token.Name.Namespace: 'bold #ansiblue',
-                    Token.Name.Variable.Magic: '#ansiblue',
-                    Token.Prompt: '#ansigreen',
-                    Token.PromptNum: '#ansibrightgreen bold',
-                    Token.OutPrompt: '#ansired',
-                    Token.OutPromptNum: '#ansibrightred bold',
-                })
+        assert name_or_cls == "legacy"
+        legacy = self.colors.lower()
 
-                # Hack: Due to limited color support on the Windows console
-                # the prompt colors will be wrong without this
-                if os.name == 'nt':
-                    style_overrides.update({
-                        Token.Prompt: '#ansidarkgreen',
-                        Token.PromptNum: '#ansigreen bold',
-                        Token.OutPrompt: '#ansidarkred',
-                        Token.OutPromptNum: '#ansired bold',
-                    })
-            elif legacy =='nocolor':
-                style_cls=_NoStyle
-                style_overrides = {}
-            else :
-                raise ValueError('Got unknown colors: ', legacy)
-        else :
-            if isinstance(name_or_cls, str):
-                style_cls = get_style_by_name(name_or_cls)
+        theme = theme_table.get(legacy, None)
+        assert theme is not None, legacy
+
+        if legacy == "nocolor":
+            style_overrides = {}
+            style_cls = _NoStyle
+        else:
+            style_overrides = {**theme.extra_style, **self.highlighting_style_overrides}
+            if theme.base is not None:
+                style_cls = get_style_by_name(theme.base)
             else:
-                style_cls = name_or_cls
-            style_overrides = {
-                Token.Prompt: '#ansigreen',
-                Token.PromptNum: '#ansibrightgreen bold',
-                Token.OutPrompt: '#ansired',
-                Token.OutPromptNum: '#ansibrightred bold',
-            }
-        style_overrides.update(self.highlighting_style_overrides)
-        style = merge_styles([
-            style_from_pygments_cls(style_cls),
-            style_from_pygments_dict(style_overrides),
-        ])
+                style_cls = _NoStyle
+
+        style = merge_styles(
+            [
+                style_from_pygments_cls(style_cls),
+                style_from_pygments_dict(style_overrides),
+            ]
+        )
 
         return style
 
@@ -729,6 +863,16 @@ class TerminalInteractiveShell(InteractiveShell):
     def color_depth(self):
         return (ColorDepth.TRUE_COLOR if self.true_color else None)
 
+    def _ptk_prompt_cont(self, width: int, line_number: int, wrap_count: int):
+        return PygmentsTokens(
+            _backward_compat_continuation_prompt_tokens(
+                self.prompts.continuation_prompt_tokens,
+                width,
+                lineno=line_number,
+                wrap_count=wrap_count,
+            )
+        )
+
     def _extra_prompt_options(self):
         """
         Return the current layout option for the current Terminal InteractiveShell
@@ -736,7 +880,7 @@ class TerminalInteractiveShell(InteractiveShell):
         def get_message():
             return PygmentsTokens(self.prompts.in_prompt_tokens())
 
-        if self.editing_mode == 'emacs':
+        if self.editing_mode == "emacs" and self.prompt_line_number_format == "":
             # with emacs mode the prompt is (usually) static, so we call only
             # the function once. With VI mode it can toggle between [ins] and
             # [nor] so we can't precompute.
@@ -751,11 +895,7 @@ class TerminalInteractiveShell(InteractiveShell):
             "lexer": IPythonPTLexer(),
             "reserve_space_for_menu": self.space_for_menu,
             "message": get_message,
-            "prompt_continuation": (
-                lambda width, lineno, is_soft_wrap: PygmentsTokens(
-                    self.prompts.continuation_prompt_tokens(width)
-                )
-            ),
+            "prompt_continuation": self._ptk_prompt_cont,
             "multiline": True,
             "complete_style": self.pt_complete_style,
             "input_processors": [
@@ -774,14 +914,13 @@ class TerminalInteractiveShell(InteractiveShell):
                     & ~IsDone()
                     & Condition(
                         lambda: isinstance(
-                            self.auto_suggest, NavigableAutoSuggestFromHistory
+                            self.auto_suggest,
+                            NavigableAutoSuggestFromHistory,
                         )
                     ),
                 ),
             ],
         }
-        if not PTK3:
-            options['inputhook'] = self.inputhook
 
         return options
 
@@ -798,34 +937,26 @@ class TerminalInteractiveShell(InteractiveShell):
         # If we don't do this, people could spawn coroutine with a
         # while/true inside which will freeze the prompt.
 
-        policy = asyncio.get_event_loop_policy()
-        old_loop = get_asyncio_loop()
-
-        # FIXME: prompt_toolkit is using the deprecated `asyncio.get_event_loop`
-        # to get the current event loop.
-        # This will probably be replaced by an attribute or input argument,
-        # at which point we can stop calling the soon-to-be-deprecated `set_event_loop` here.
-        if old_loop is not self.pt_loop:
-            policy.set_event_loop(self.pt_loop)
-        try:
-            with patch_stdout(raw=True):
+        with patch_stdout(raw=True):
+            if self._use_asyncio_inputhook:
+                # When we integrate the asyncio event loop, run the UI in the
+                # same event loop as the rest of the code. don't use an actual
+                # input hook. (Asyncio is not made for nesting event loops.)
+                asyncio_loop = get_asyncio_loop()
+                text = asyncio_loop.run_until_complete(
+                    self.pt_app.prompt_async(
+                        default=default, **self._extra_prompt_options()
+                    )
+                )
+            else:
                 text = self.pt_app.prompt(
                     default=default,
-                    **self._extra_prompt_options())
-        finally:
-            # Restore the original event loop.
-            if old_loop is not None and old_loop is not self.pt_loop:
-                policy.set_event_loop(old_loop)
+                    inputhook=self._inputhook,
+                    **self._extra_prompt_options(),
+                )
 
         return text
 
-    def enable_win_unicode_console(self):
-        # Since IPython 7.10 doesn't support python < 3.6 and PEP 528, Python uses the unicode APIs for the Windows
-        # console by default, so WUC shouldn't be needed.
-        warn("`enable_win_unicode_console` is deprecated since IPython 7.10, does not do anything and will be removed in the future",
-             DeprecationWarning,
-             stacklevel=2)
-
     def init_io(self):
         if sys.platform not in {'win32', 'cli'}:
             return
@@ -849,16 +980,14 @@ class TerminalInteractiveShell(InteractiveShell):
             for cmd in ('clear', 'more', 'less', 'man'):
                 self.alias_manager.soft_define_alias(cmd, cmd)
 
-
     def __init__(self, *args, **kwargs) -> None:
-        super(TerminalInteractiveShell, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self._set_autosuggestions(self.autosuggestions_provider)
         self.init_prompt_toolkit_cli()
         self.init_term_title()
         self.keep_running = True
         self._set_formatter(self.autoformatter)
 
-
     def ask_exit(self):
         self.keep_running = False
 
@@ -906,14 +1035,25 @@ class TerminalInteractiveShell(InteractiveShell):
 
         self._atexit_once()
 
-
     _inputhook = None
     def inputhook(self, context):
+        warn(
+            "inputkook seem unused, and marked for deprecation/Removal as of IPython 9.0. "
+            "Please open an issue if you are using it.",
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
         if self._inputhook is not None:
             self._inputhook(context)
 
-    active_eventloop = None
-    def enable_gui(self, gui=None):
+    active_eventloop: Optional[str] = None
+
+    def enable_gui(self, gui: Optional[str] = None) -> None:
+        if gui:
+            from ..core.pylabtools import _convert_gui_from_matplotlib
+
+            gui = _convert_gui_from_matplotlib(gui)
+
         if self.simple_prompt is True and gui is not None:
             print(
                 f'Cannot install event loop hook for "{gui}" when running with `--simple-prompt`.'
@@ -928,43 +1068,28 @@ class TerminalInteractiveShell(InteractiveShell):
             return
 
         if self._inputhook is not None and gui is not None:
-            print(
-                f"Shell is already running a gui event loop for {self.active_eventloop}. "
+            newev, newinhook = get_inputhook_name_and_func(gui)
+            if self._inputhook == newinhook:
+                # same inputhook, do nothing
+                self.log.info(
+                    f"Shell is already running the {self.active_eventloop} eventloop. Doing nothing"
+                )
+                return
+            self.log.warning(
+                f"Shell is already running a different gui event loop for {self.active_eventloop}. "
                 "Call with no arguments to disable the current loop."
             )
             return
         if self._inputhook is not None and gui is None:
             self.active_eventloop = self._inputhook = None
 
-        if gui and (gui not in {"inline", "webagg"}):
+        if gui and (gui not in {None, "webagg"}):
             # This hook runs with each cycle of the `prompt_toolkit`'s event loop.
             self.active_eventloop, self._inputhook = get_inputhook_name_and_func(gui)
         else:
             self.active_eventloop = self._inputhook = None
 
-        # For prompt_toolkit 3.0. We have to create an asyncio event loop with
-        # this inputhook.
-        if PTK3:
-            import asyncio
-            from prompt_toolkit.eventloop import new_eventloop_with_inputhook
-
-            if gui == 'asyncio':
-                # When we integrate the asyncio event loop, run the UI in the
-                # same event loop as the rest of the code. don't use an actual
-                # input hook. (Asyncio is not made for nesting event loops.)
-                self.pt_loop = get_asyncio_loop()
-                print("Installed asyncio event loop hook.")
-
-            elif self._inputhook:
-                # If an inputhook was set, create a new asyncio event loop with
-                # this inputhook for the prompt.
-                self.pt_loop = new_eventloop_with_inputhook(self._inputhook)
-                print(f"Installed {self.active_eventloop} event loop hook.")
-            else:
-                # When there's no inputhook, run the prompt in a separate
-                # asyncio event loop.
-                self.pt_loop = asyncio.new_event_loop()
-                print("GUI event loop hook disabled.")
+        self._use_asyncio_inputhook = gui == "asyncio"
 
     # Run !system commands directly, not through pipes, so terminal programs
     # work correctly.
